// Step 3: Combine firm description files and save them in the appropriate country folder

* 0) Preliminaries
*** Cleaning
	clear
	cls
	set more off
	set rmsg on																	// Keeps track of length of process
	
*** Install codes
   ssc install fs
   ssc install confirmdir
   ssc install chunky
   
*** Directories
global unzip "C:\Program Files\7-Zip\7z"				      					// Location of 7z executable program
global loc "CHANGE TO RELEVANT DRIVE"											// CHANGE DRIVE WHERE FORLDERS ARE LOCATED
global raw "$loc\financials\zip" 												// Location of zip files (SUGGESTION: COPY INTO THIS FOLDER ONLY THE ZIP FILES THAT YOU WANT TO CONVERT)
global txt "$loc\financial\txt"													// Output text files
global dta "$loc\financials\dta"												// Output data files 

global codes "$loc\processing_codes"											// Locataion to save log files

*** Create Country Folders (the first one is for the unstacked files)
use "$loc\countries_ISO.dta", clear
levelsof country, local(cntry)
foreach c of local cntry {
	confirmdir "$dta\intermediate_countries\\`c'"
	if r(confirmdir) == "170" {
		mkdir "$dta\intermediate_countries\\`c'"
	}
}

*** Create Country Folders (the second set is for stacked final files)
use "$loc\countries_ISO.dta", clear
levelsof country, local(cntry)
foreach c of local cntry {
	confirmdir "$dta\appended_countries\\`c'"
	if r(confirmdir) == "170" {
		mkdir "$dta\appended_countries\\`c'"
	}
}

* 1) Save intermediate files by country
fs "$dta\intermediate\*.dta"
	return list
set obs 1 
	gen fn = ""
	gen id = 0
foreach f in `r(files)' {
	insobs 1
	replace id = id[_n-1] + 1 if _n > 1
	local name `f'
	replace fn = "`name'" if _n == _N
}
drop if id == 0
keep if strpos(fn,"industryglobalfinancialsandratiosusd") > 0
split fn, p(".") gen(fname)
	keep fname1 id
	rename fname1 fname
	gen string = substr(fname,-2,2)
	keep id string
		drop id
		gen id = _n
	 disp "List of files in processing loop"
save "$loc\financials\files_all.dta", replace
	qui sum id
disp "Beginning to loop 1 through files: saving the data"	
forval i = `r(min)'(1)`r(max)'{
	use "$loc\financials\files_all.dta", clear
	keep if id == `i'
	global file `=fname[1]'
	
	use "$dta\\$file.dta", clear
	// Save individual country files
	levelsof ISO, local(cntry)
	foreach c of local cntry {
	preserve
		disp "Processing `c' portion of $file"
		keep if ISO == "`c'"
		drop ISO
		save "$dta\intermediate_countries\\`c'\\`c'_$file.dta", replace
	restore
	}		
	erase "$dta\\$file.dta"
}

erase "$loc\financials\files_all.dta"

foreach c of local cntry {
	* 2) Append the files
	clear
	fs "$dta\intermediate_countries\\`c'\\*.dta"
		return list
	set obs 1 
		gen fn = ""
		gen id = 0
	foreach f in `r(files)' {
		insobs 1
		replace id = id[_n-1] + 1 if _n > 1
		local name `f'
		replace fn = "`name'" if _n == _N
	}
	drop if id == 0
	keep if strpos(fn,"industryglobalfinancialsandratiosusd") > 0
	save "$loc\financials\process.dta", replace
	
	*** This sequence determines if the country has financials files and generates a list of the fiancials files that need to be processed
	count
	if `r(N)' != 0 {
		split fn, p(".") gen(fname)
			keep fname1 id
			rename fname1 fname
			gen string = substr(fname,-2,2)
			keep id string
				drop id
				gen id = _n
			 disp "List of files in processing loop"
		save "$loc\financials\process.dta", replace
			qui sum id
			return list
			count
	}
	*** If country has financials files, then start with the first file in the sequence
	if `r(N)' != 0 {
		noisily keep if id == 1
		global st `=string[1]'
		disp "$st"
		use "$dta\intermediate_countries\\`c'\\`c'_industryglobalfinancialsandratiosusd00$st.txt.dta", clear
		preserve
			use "$loc\financials\process.dta", clear
			qui sum id
			return list
		restore
	}
	*** If country featured only in one file, finish the file
	if `r(N)' == 1 {
		drop if bvdid == ""
		duplicates drop		
		save "$dta\appended_countries\\`c'\\Financial_`c'.dta", replace
	}
	*** If country featured in multiple files, loop over all the files
	if `r(N)' > 1 {
		forval i = 2(1)`r(max)' {
			preserve
				use "$loc\financials\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			append using "$dta\intermediate_countries\\`c'\\`c'_industryglobalfinancialsandratiosusd00$st.txt.dta"
		}
		drop if bvdid == ""
		duplicates drop
		save "$dta\appended_countries\\`c'\\Financial_`c'.dta", replace
	}

	*** NOTE: THIS STEP IS OPTIONAL, IT ERASES ALL OF THE INTERMDIATE FILES (WILL LEAVE THE INTERMEDIATE FOLDER EMPTY)
	use "$loc\financials\process.dta", clear
		qui sum id
		return list
	if `r(N)' == 1 {
		use "$loc\financials\process.dta", clear
		noisily keep if id == 1
		global st `=string[1]'	
		disp "$st"	
		erase "$dta\intermediate_countries\\`c'\\`c'_industryglobalfinancialsandratiosusd00$st.txt.dta"	
	}
	if `r(N)' > 1 {
		forval i = `r(min)'(1)`r(max)' {
			preserve
				use "$loc\financials\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			erase "$dta\intermediate_countries\\`c'\\`c'_industryglobalfinancialsandratiosusd00$st.txt.dta"
		}
	}
	
	erase "$loc\financials\process.dta"
}
